@@ -1,6 +1,6 @@
 /*
 	conffile.cc	Configuration File Functions
-	Copyright (c) 2000, 2001, 2002 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2002, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -276,6 +276,7 @@ struct	ConfigValue {
 	bool	KcdConfig::*cfgBoolPtr;
 	DirList	* KcdConfig::*cfgStringListPtr;
 	AttrConfig KcdConfig::*cfgAttrPtr;
+	KeyBindingType KcdConfig::*cfgKeyPtr;
 };
 
 /*************************************************************************
@@ -378,6 +379,77 @@ void	PrintUnsigned(const char *cmd, const ConfigValue &cfg)
 	cout << cmd << ": " << allConfig[configProfile].*cfg.cfgUIntPtr << '\n';
 }
 
+/*************************************************************************
+	Command processing/printing functions - Key binding.
+*************************************************************************/
+
+struct	KeyBindingMap {
+	char		*name;
+	KeyBindingType	value;
+};
+
+#define NUM_KEY_BINDING	4
+
+KeyBindingMap	keyBindingMap[NUM_KEY_BINDING] = {
+	{ "default", keyBindingDefault },
+	{ "kcd", keyBindingDefault },
+	{ "vi", keyBindingVi },
+	{ "emacs", keyBindingEmacs }
+};
+
+
+void	SetKeyBinding(const ConfigValue &cfg, KeyBindingType k)
+{
+	allConfig[configProfile].*cfg.cfgKeyPtr = k;
+}
+
+void	ConfigKeyBinding(const char *cmd, char *buf, size_t from, size_t len,
+		  const ConfigValue &cfg)
+{
+	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
+
+	size_t	i;
+	size_t	tok_end = SeekConfigTokenEnd(buf, from, len);
+
+	for (i = 0; i < NUM_KEY_BINDING; ++i) {
+		size_t key_len = strlen(keyBindingMap[i].name);
+		if (tok_end-from == key_len 
+		    && CompareStringCase(keyBindingMap[i].name, buf+from, key_len) == 0)
+			break;
+	}
+	if (i == NUM_KEY_BINDING)
+			// Throw exception
+		PrintErrorMessage(_("invalid attribute"), cmd, buf);
+
+	from = tok_end;
+	from = SeekNonSpace(buf, from, len);
+	if (from != len)
+		ExtraChar(cmd, buf);
+
+	KeyBindingType k = keyBindingMap[i].value;
+		
+			// Note: error has already been reported
+
+	SetKeyBinding(cfg, k);
+}
+
+void	PrintKeyBinding(const char *cmd, const ConfigValue &cfg)
+{
+	cout << cmd << ": ";
+	switch (allConfig[configProfile].*cfg.cfgKeyPtr) {
+		case keyBindingDefault:
+			cout << "Default";
+			break;
+		case keyBindingVi:
+			cout << "vi";
+			break;
+		case keyBindingEmacs:
+			cout << "emacs";
+			break;
+	}
+	cout << '\n';
+}
+
 /*************************************************************************
 	Command processing/printing functions - String list.
 *************************************************************************/
@@ -444,7 +516,7 @@ void	ConfigStringList(const char *cmd, char *buf, size_t from, size_t len,
 					
 	string	expanded_dir;
 	try {
-		expanded_dir = ExpandDirectory(buf+from, 1);
+		expanded_dir = ExpandDirectory(buf+from, true);
 	}
 	catch (ErrorGenericSyntax &e) {
 		PrintErrorMessage(e.what(), cmd, buf);
@@ -530,7 +602,7 @@ void	ConfigClearStringList(const char *cmd, char *buf, size_t from, size_t len,
 
 		string	expanded_dir;
 		try {
-			expanded_dir = ExpandDirectory(buf+from, 1);
+			expanded_dir = ExpandDirectory(buf+from, true);
 		}
 		catch (ErrorGenericSyntax &e) {
 			PrintErrorMessage(e.what(), cmd, buf);
@@ -602,7 +674,7 @@ void	ConfigColorAttr(const char *cmd, char *buf, size_t from, size_t len,
 	ATTR_TYPE colorAttr = A_NORMAL;
 	COLOR_TYPE colorForeground = COLOR_WHITE;
 	COLOR_TYPE colorBackground = COLOR_BLACK;
-	int i, tok_end;
+	size_t i, tok_end;
 	for ( ; ; ) {
 		tok_end = SeekConfigTokenEnd(buf, from, len);
 		for (i = 0; i < NUM_ATTR; ++i) {
@@ -696,7 +768,7 @@ void	PrintColorAttr(const char *cmd, const ConfigValue &cfg)
 		}
 	}
 
-	for (int i = 0; i < NUM_COLOR; ++i) {
+	for (size_t i = 0; i < NUM_COLOR; ++i) {
 		if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorForeground == colorMap[i].value) {
 			cout << colorMap[i].name << ' ';
 			break;
@@ -705,7 +777,7 @@ void	PrintColorAttr(const char *cmd, const ConfigValue &cfg)
 
 	cout << "on ";
 
-	for (int i = 0; i < NUM_COLOR; ++i) {
+	for (size_t i = 0; i < NUM_COLOR; ++i) {
 		if ((allConfig[configProfile].*cfg.cfgAttrPtr).colorBackground == colorMap[i].value) {
 			cout << colorMap[i].name;
 			break;
@@ -729,7 +801,7 @@ void	ConfigColor(const char *cmd, char *buf, size_t from, size_t len,
 	from = SeekPastEqualSignOrThrow(cmd, buf, from, len);
 
 	COLOR_TYPE color = COLOR_BLACK;		// Avoid compiler warning
-	int i, tok_end;
+	size_t i, tok_end;
 
 	tok_end = SeekConfigTokenEnd(buf, from, len);
 	for (i = 0; i < NUM_COLOR; ++i) {
@@ -783,8 +855,8 @@ void	ConfigBWAttr(const char *cmd, char *buf, size_t from, size_t len,
 	bool found_normal = false;
 	ATTR_TYPE bwAttr = A_NORMAL;
 	for ( ; ; ) {
-		int tok_end = SeekConfigTokenEnd(buf, from, len);
-		int i;
+		size_t tok_end = SeekConfigTokenEnd(buf, from, len);
+		size_t i;
 		for (i = 0; i < NUM_ATTR; ++i) {
 			size_t attr_len = strlen(attrMap[i].name);
 			if (tok_end-from == attr_len 
@@ -850,60 +922,64 @@ struct	ConfigInfo {
 
 ConfigFunc cfgFuncYesNo = { ConfigYesNo, PrintYesNo };
 ConfigFunc cfgFuncUnsigned = { ConfigUnsigned, PrintUnsigned };
+ConfigFunc cfgFuncKeyBinding = { ConfigKeyBinding, PrintKeyBinding };
 ConfigFunc cfgFuncStringList = { ConfigStringList, PrintStringList };
 ConfigFunc cfgFuncClearStringList = { ConfigClearStringList, 0 };
 ConfigFunc cfgFuncColorAttr = { ConfigColorAttr, PrintColorAttr };
 ConfigFunc cfgFuncColor = { ConfigColor, PrintColor };
 ConfigFunc cfgFuncBWAttr = { ConfigBWAttr, PrintBWAttr };
 
-#define NUM_CONFIG_INFO	39
+#define NUM_CONFIG_INFO	41
 ConfigInfo configInfo[NUM_CONFIG_INFO] = {
-	{ "QuietFullScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietFull, 0, 0 } },
-	{ "QuietSmartScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietSmart, 0, 0 } },
-	{ "QuietPartialScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietPartial, 0, 0 } },
-	{ "AutoScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgAutoScan, 0, 0 } },
-	{ "SpaceSelect", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSpaceSelect, 0, 0 } },
-	{ "SortTree", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSortTree, 0, 0 } },
-	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgCaseSensitiveSort, 0, 0 } },
-	{ "GraphicChar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGraphicChar, 0, 0 } },
-	{ "ScrollBar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgScrollBar, 0, 0 } },
-	{ "GlobDot", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobDot, 0, 0 } },
-	{ "GlobPath", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobPath, 0, 0 } },
+	{ "QuietFullScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietFull, 0, 0, 0 } },
+	{ "QuietSmartScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietSmart, 0, 0, 0 } },
+	{ "QuietPartialScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgQuietPartial, 0, 0, 0 } },
+	{ "AutoScan", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgAutoScan, 0, 0, 0 } },
+	{ "SpaceSelect", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSpaceSelect, 0, 0, 0 } },
+	{ "SortTree", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgSortTree, 0, 0, 0 } },
+	{ "CaseSensitiveSort", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgCaseSensitiveSort, 0, 0, 0 } },
+	{ "GraphicChar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGraphicChar, 0, 0, 0 } },
+	{ "ScrollBar", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgScrollBar, 0, 0, 0 } },
+	{ "GlobDot", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobDot, 0, 0, 0 } },
+	{ "GlobPath", &cfgFuncYesNo, { 0, 0, &KcdConfig::cfgGlobPath, 0, 0, 0 } },
 
-	{ "ShowListThreshold", &cfgFuncUnsigned, { &KcdConfig::cfgShowListThreshold, 0, 0, 0, 0 } },
+	{ "KeyBinding", &cfgFuncKeyBinding, { 0, 0, 0, 0, 0, &KcdConfig::cfgKey } },
 
-	{ "StartDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0 } },
-	{ "SkipDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0 } },
+	{ "ShowListThreshold", &cfgFuncUnsigned, { &KcdConfig::cfgShowListThreshold, 0, 0, 0, 0, 0 } },
+	{ "FuzzySize", &cfgFuncUnsigned, { &KcdConfig::cfgFuzzySize, 0, 0, 0, 0, 0 } },
 
-	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0 } },
-	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0 } },
+	{ "StartDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0 } },
+	{ "SkipDir", &cfgFuncStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0 } },
 
-	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal } },
-	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal } },
-	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink } },
-	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink } },
-	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight } },
-	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight } },
-	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic } },
-	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic } },
-	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic } },
-	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic } },
+	{ "ClearStartDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgStartDir, 0, 0 } },
+	{ "ClearSkipDir", &cfgFuncClearStringList, { 0, 0, 0, &KcdConfig::cfgSkipDir, 0, 0 } },
 
-	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle } },
-	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle } },
-	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL } },
-	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL } },
-	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore } },
-	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore } },
+	{ "NormalColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0 } },
+	{ "NormalBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrNormal, 0 } },
+	{ "DirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0 } },
+	{ "DirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLink, 0 } },
+	{ "HighlightDirColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0 } },
+	{ "HighlightDirBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlight, 0 } },
+	{ "SymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0 } },
+	{ "SymLinkBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrLinkItalic, 0 } },
+	{ "HighlightSymLinkColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0 } },
+	{ "HighlightSymLinkAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrHighlightItalic, 0 } },
 
-	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow } },
-	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow } },
-	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock } },
-	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock } },
-	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar } },
-	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar } },
+	{ "TitleColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0 } },
+	{ "TitleBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrTitle, 0 } },
+	{ "StatusColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0 } },
+	{ "StatusBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrURL, 0 } },
+	{ "MoreColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0 } },
+	{ "MoreBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrMore, 0 } },
 
-	{ "DefaultBackground", &cfgFuncColor, { 0, &KcdConfig::cfgDefaultBackground, 0, 0, 0 } }
+	{ "ScrollArrowColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0 } },
+	{ "ScrollArrowBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollArrow, 0 } },
+	{ "ScrollBlockColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0 } },
+	{ "ScrollBlockBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBlock, 0 } },
+	{ "ScrollBarColorAttr", &cfgFuncColorAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0 } },
+	{ "ScrollBarBWAttr", &cfgFuncBWAttr, { 0, 0, 0, 0, &KcdConfig::cfgAttrScrollBar, 0 } },
+
+	{ "DefaultBackground", &cfgFuncColor, { 0, &KcdConfig::cfgDefaultBackground, 0, 0, 0, 0 } }
 };
 
 /*************************************************************************
