@@ -1,6 +1,6 @@
 /*
 	confobj.cc	Configuration Class
-	Copyright (c) 2000, 2002 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2002, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,8 @@ KcdConfig::KcdConfig() {
 	cfgCaseSensitiveSort = 0;	// Equals 1 for case-sensitive sort
 	cfgShowListThreshold = 4;	// When match more than the specified
 					// number, show matched dir as a list
+	cfgFuzzySize = 10;		// Number of directory kept for
+					// fuzzy searching
 	cfgGraphicChar = 1;		// Enable line-drawing chars, etc.
 
 #ifdef KCD_NC_GOOD_SCROLL
@@ -46,6 +48,8 @@ KcdConfig::KcdConfig() {
 	cfgGlobDot = 0;
 	cfgGlobPath = 0;
 
+	cfgKey = keyBindingDefault;
+
 	SetAttr(cfgAttrNormal, COLOR_WHITE, DEFAULT_COLOR, A_NORMAL, A_NORMAL);
 	SetAttr(cfgAttrLink, COLOR_GREEN, DEFAULT_COLOR, A_NORMAL, A_UNDERLINE);
 	SetAttr(cfgAttrHighlight, COLOR_CYAN, COLOR_BLUE, A_NORMAL, A_REVERSE);
@@ -91,12 +95,15 @@ KcdConfig &KcdConfig::operator=(const KcdConfig &c)
 	cfgSortTree = c.cfgSortTree;
 	cfgCaseSensitiveSort = c.cfgCaseSensitiveSort;
 	cfgShowListThreshold = c.cfgShowListThreshold;
+	cfgFuzzySize = c.cfgFuzzySize;
 	cfgGraphicChar = c.cfgGraphicChar;
 	cfgScrollBar = c.cfgScrollBar;
 
 	cfgGlobDot = c.cfgGlobDot;
 	cfgGlobPath = c.cfgGlobPath;
 
+	cfgKey = c.cfgKey;
+
 	cfgAttrNormal = c.cfgAttrNormal;
 	cfgAttrLink = c.cfgAttrLink;
 	cfgAttrHighlight = c.cfgAttrHighlight;
