@@ -1,6 +1,6 @@
 /*
 	confobj.h	Configuration Class Header
-	Copyright (c) 2000, 2001, 2002 Kriang Lerdsuwanakij
+	Copyright (c) 2000, 2001, 2002, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -54,6 +54,12 @@ typedef sptr<DirConfig>		DirItem;
 #define SCAN_MODE_SMART		1
 #define SCAN_MODE_PARTIAL	2
 
+enum KeyBindingType {
+	keyBindingDefault = 0,
+	keyBindingVi,
+	keyBindingEmacs
+};
+
 extern string	confLocalFile;
 
 #define DEFAULT_COLOR		INT_MAX		/* Must be the same type as
@@ -81,7 +87,10 @@ struct KcdConfig {
 	bool	cfgGlobDot;
 	bool	cfgGlobPath;
 
+	KeyBindingType	cfgKey;
+
 	unsigned	cfgShowListThreshold;
+	unsigned	cfgFuzzySize;
 
 	AttrConfig	cfgAttrNormal;
 	AttrConfig	cfgAttrLink;
