@@ -55,11 +55,13 @@ string	my_strftime(const char *format, const struct tm *tm)
 
 string	html_unreadable;
 string	html_skipped;
+string	html_outside_tree;
 string	html_not_in_file;
 string	html_not_found;
 
 size_t	html_unreadable_width;
 size_t	html_skipped_width;
+size_t	html_outside_tree_width;
 size_t	html_not_in_file_width;
 size_t	html_not_found_width;
 
@@ -67,11 +69,13 @@ void	GenerateHTMLStrings()
 {
 	html_unreadable = _(" [unreadable]");
 	html_skipped = _(" [skipped]");
+	html_outside_tree = _(" [outside tree]");
 	html_not_in_file = _(" [*]");
 	html_not_found = _(" [not found]");
 
 	html_unreadable_width = GetStringWidth(html_unreadable);
 	html_skipped_width = GetStringWidth(html_skipped);
+	html_outside_tree_width = GetStringWidth(html_outside_tree);
 	html_not_in_file_width = GetStringWidth(html_not_in_file);
 	html_not_found_width = GetStringWidth(html_not_found);
 }
@@ -82,6 +86,17 @@ void	GenerateHTMLStrings()
 
 size_t	htmlRow, htmlCol;
 
+void	ReplaceControlChar(string &s)
+{
+	for (size_t i = 0; i < s.size(); ++i) {
+						// Characters interpreted by
+						// DrawString
+		if (s[i] == '\t' || s[i] == '\r' || s[i] == '\b' || s[i] == '\0'
+		    || s[i] == '\n')
+			s[i] = '?';
+	}
+}
+
 class HyperTreeDraw : public HyperDraw {
 	private:
 		vector<size_t>	barStack;	// Positions of vertical bars
@@ -131,12 +146,24 @@ void	HyperTreeDraw::GenerateSymLinkHTML(HyperDocument &html, WINDOW *pad,
 
 		html.DrawSetItalic(pad);
 
+		tagBegin = true;
+	}
+	else if (d->isLinkDestSkip == 1) {
+						// Write tags
+		html.DrawAName(pad, new_dir);
+
+		html.DrawACallBegin(pad, new_dir);
+
+		html.DrawSetItalic(pad);
+
 		tagBegin = true;
 	}
 
 			    			// Write dir. name
-	html.DrawString(pad, d->GetNameStr());
-	curCol += GetStringWidth(d->GetNameStr());
+	string s = d->GetNameStr();
+	ReplaceControlChar(s);
+	html.DrawString(pad, s);
+	curCol += GetStringWidth(s);
 			
 			    			// Write graphical " -> "
 	html.DrawChar(pad, ' ');
@@ -146,8 +173,10 @@ void	HyperTreeDraw::GenerateSymLinkHTML(HyperDocument &html, WINDOW *pad,
 	curCol += 4;
 
 			    			// Write linked dir.
-	html.DrawString(pad, d->GetLinkStr());
-	curCol += GetStringWidth(d->GetLinkStr());
+	s = d->GetLinkStr();
+	ReplaceControlChar(s);
+	html.DrawString(pad, s);
+	curCol += GetStringWidth(s);
 
 	switch(d->isLinkDestSkip) {
 		case 0:
@@ -157,8 +186,8 @@ void	HyperTreeDraw::GenerateSymLinkHTML(HyperDocument &html, WINDOW *pad,
 			}
 			break;
 		case 1:
-			html.DrawString(pad, html_skipped);
-			curCol += html_skipped_width;
+			html.DrawString(pad, html_outside_tree);
+			curCol += html_outside_tree_width;
 			break;
 		case 2:
 			html.DrawString(pad, html_not_in_file);
@@ -186,10 +215,14 @@ void	HyperTreeDraw::GenerateDirHTML(HyperDocument &html, WINDOW *pad,
 		html.DrawAName(pad, new_dir);
 		html.DrawACallBegin(pad, new_dir);
 	}
-			    		
+//ofstream ofs("xx", ios::app);
+//if (!pad)
+//ofs << htmlRow << ' ' << htmlCol << ' ' << curCol << ' ' << d->GetNameStr() << '\n';
 			    			// Write dir. name
-	html.DrawString(pad, d->GetNameStr());
-	curCol += GetStringWidth(d->GetNameStr());
+	string s = d->GetNameStr();
+	ReplaceControlChar(s);
+	html.DrawString(pad, s);
+	curCol += GetStringWidth(s);
 			    		
 	if (d->isUnreadable != 1) {		// Exec permission
 			    			// Dir. can be accessed
@@ -360,10 +393,15 @@ void	HyperTreeDraw::Draw(HyperDocument &html, WINDOW *pad)
 		}
 	}
 
-	if (curCol != 0)		// Not end with new line
+	if (curCol != 0) {		// Not end with new line
+		html.DrawChar(pad, '\n');
 		NewLineUpdateRowCol();	// One more line needed
+	}
 
 	if (! pad) {
+//ofstream ofs("xx", ios::app);
+//if (!pad)
+//ofs << htmlRow << ' ' << htmlCol << ' ' << '\n';
 		html.DrawSetSize(htmlRow, htmlCol);
 	}
 	else {
@@ -379,10 +417,11 @@ struct MatchInfo {
 	string			name;
 	sptr<DirectoryEntry>	dir;
 	bool			is_curdir;
+	unsigned		error;
 
 	MatchInfo(const string &name_, sptr<DirectoryEntry> &dir_,
-		  bool is_curdir_) : 
-		name(name_), dir(dir_), is_curdir(is_curdir_)
+		  bool is_curdir_, unsigned error_ = 0) : 
+		name(name_), dir(dir_), is_curdir(is_curdir_), error(error_)
 	{
 	}
 };
@@ -415,6 +454,109 @@ struct MatchInfoIsNotBookmarkedNorCurDir {
 	Find Match
 *************************************************************************/
 
+// The add_curdir feature isn't handled here fuzzy find because it
+// doesn't work well with error sorting order.
+void   FuzzyRecursiveFindMatchDir(list<MatchInfo> &match_list,
+				  sptr<DirectoryEntry> &dir, 
+				  const string &str,
+				  const string &current_dir)
+{
+	unsigned subStrLen;
+	unsigned StrLen;
+	unsigned cnt1, cnt2, TotCnt;
+	unsigned TmpErr;
+	int TmpVal;
+
+	subStrLen=str.size();
+	if (dir->IsDir() && dir->isUnreadable != 1) {
+		string mstr = dir->GetNameStr();
+		StrLen = mstr.size();
+		for (size_t i = 0; i < StrLen; ++i)
+			mstr[i] = tolower(mstr[i]);
+
+		if (subStrLen < StrLen) {
+			TotCnt = StrLen-subStrLen+1;
+			unsigned err = static_cast<unsigned>(-1);
+			for (cnt1=0; cnt1 < TotCnt; cnt1++) {
+				TmpErr = 0;
+				for (cnt2 = 0; cnt2 < subStrLen; cnt2++) {
+					TmpVal = mstr[cnt1+cnt2-1]
+						 - tolower(str[cnt2]);
+					if (TmpVal < 0)
+						TmpVal =- TmpVal;
+					TmpErr += TmpVal * (subStrLen-cnt2+1);
+				}
+				TmpErr += cnt1*2;
+				TmpErr += StrLen-subStrLen;
+				for (cnt2 = 0; cnt2 < subStrLen; cnt2++) {
+					if (mstr.find(tolower(str[cnt2])) == mstr.npos)
+						TmpErr += 100;
+				}
+
+				if (TmpErr < err)
+					err = TmpErr;
+			}
+
+			if (match_list.size()
+			    < static_cast<size_t>(kcdConfig.cfgFuzzySize)) {
+				list<MatchInfo>::iterator iter = match_list.begin();
+				for ( ; iter != match_list.end(); ++iter) {
+					if ((*iter).error > err)
+						break;
+				}
+					// Insert before iter
+				match_list.insert(iter,
+						  MatchInfo(current_dir, dir, false, err));
+			}
+					// We have a fully populated list
+			else if (match_list.back().error > err) {
+				match_list.pop_back();
+
+				list<MatchInfo>::iterator iter = match_list.begin();
+				for ( ; iter != match_list.end(); ++iter) {
+					if ((*iter).error > err)
+						break;
+				}
+					// Insert before iter
+				match_list.insert(iter,
+						  MatchInfo(current_dir, dir, false, err));
+//cout << current_dir << ' ' << err << '\n';
+			}
+		}
+	}
+
+	for (sptr_list<DirectoryEntry>::iterator iter = dir->subDir.begin();
+	     iter != dir->subDir.end(); ++iter) {      
+
+						// For dirs
+		if ((*iter)->IsDir()) {
+			string new_dir = current_dir;
+			if (new_dir[new_dir.size()-1] != '/')
+				new_dir += '/';
+			new_dir += (*iter)->GetNameStr();
+
+			FuzzyRecursiveFindMatchDir(match_list, *iter, str, new_dir);
+		}
+	}
+}
+
+size_t FuzzyFindMatchDir(list<MatchInfo> &match_list, const string &linktext)
+{
+	match_list.clear();
+
+	if (linktext.size() == 0)		// In case getcwd fails
+		return 0;
+
+	for (sptr_list<DirectoryEntry>::iterator iter = dirTree.begin();
+	     iter!=dirTree.end(); ++iter) {
+		string new_dir = (*iter)->GetNameStr();
+		FuzzyRecursiveFindMatchDir(match_list, *iter, linktext,
+					   new_dir);
+	}
+
+	return match_list.size();
+}
+
 void	RecursiveFindMatchDir(list<MatchInfo> &match_list, sptr<DirectoryEntry> &dir, 
 			      const string &str, const string &current_dir,
 			      bool add_curdir)
@@ -461,6 +603,11 @@ size_t	FindMatchDir(list<MatchInfo> &match_list, const string &linktext,
 				      add_curdir);
 	}
 	
+	if (kcdConfig.cfgFuzzySize &&
+	    (match_list.size() == 0
+	     || match_list.size() == 1 && match_list.front().is_curdir == true))
+		return FuzzyFindMatchDir(match_list, linktext);
+
 	return match_list.size();
 }
 
@@ -530,7 +677,9 @@ HyperListDraw::HyperListDraw(list<MatchInfo> &match_list_) :
 	max_length = 0;
 	for (list<MatchInfo>::iterator iter = match_list.begin();
 	     iter != match_list.end(); iter++) {
-		size_t size = GetStringWidth(iter->name);
+		string s = iter->name;
+		ReplaceControlChar(s);
+		size_t size = GetStringWidth(s);
 		if (size > max_length)
 			max_length = size;
 	}
@@ -583,7 +732,9 @@ void	HyperListDraw::Draw(HyperDocument &html, WINDOW *pad)
 			}
 			html.DrawString(pad, s);
 
-			html.DrawString(pad, iter->name);
+			string ss = iter->name;
+			ReplaceControlChar(ss);
+			html.DrawString(pad, ss);
 
 			html.DrawChar(pad, ' ');
 		
@@ -873,7 +1024,7 @@ void	DirListCD(list<MatchInfo> &match_list, const string &init_pos,
 	fclose(file);
 
 	if (match_list.size() > adjust+1)	// More than one matched dir
-	cout << _("change to ") << (*iter).name << '\n';
+		cout << _("change to ") << (*iter).name << '\n';
 }
 
 void	CD(const string &linktext, bool entire_tree)
