@@ -1,6 +1,6 @@
 /*
 	khdoc.cc	Hypertext Document Classes
-	Copyright (c) 1996-9,2000-2 Kriang Lerdsuwanakij
+	Copyright (c) 1996-9,2000-2, 2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -91,6 +91,8 @@
 # endif
 #endif
 
+#include CXX__HEADER_fstream
+
 // #define DEBUG_FORMAT_TEXT
 
 /*************************************************************************
@@ -1985,6 +1987,8 @@ void	HyperDocument::DrawString(WINDOW *pad, const char *s)
 		if (procA)
 			procA->linkText += s;
 
+//ofstream ofs("xx", ios::app);
+//ofs << curRow << ' ' << curColumn << ' ' << s << '\n';
 #ifdef USE_UTF8_MODE
 // FIXME: Handle wide/combining chars, UTF-8 encoding
 //		if (0) {
