@@ -21,6 +21,7 @@
 #include "scandir.h"
 
 #include CXX__HEADER_iomanip
+#include CXX__HEADER_fstream
 #include CXX__HEADER_algorithm
 #include CXX__HEADER_cerrno
 
@@ -117,7 +118,7 @@ void	ResolveSymLink(sptr<DirectoryEntry> &d, const string &current_dir)
 			}
 			d->UpdateLinkDest(linked_dir);
 
-			FindDirInfo *dir_info = FindDir(linked_dir, false);
+			FindDirInfo *dir_info = FindDir(linked_dir, false, false);
 			if (dir_info)
 				d->isLinkDestSkip = 0;
 			else
