@@ -1,6 +1,6 @@
 /*
 	strmisc.cc	String Functions
-	Copyright (c) 1997,1998,1999,2000,2001 Kriang Lerdsuwanakij
+	Copyright (c) 1997,1998,1999,2000,2001,2003 Kriang Lerdsuwanakij
 	email:		lerdsuwa@users.sourceforge.net
 
 	This program is free software; you can redistribute it and/or modify
@@ -139,7 +139,7 @@ string	QuoteShellChars(const string &src)	// Remove quotes
 	return dest;
 }
 
-string	UnquoteShellChars(const string &src)	// Remove quotes
+string	UnquoteShellChars(const string &src, bool quoteGlob)	// Remove quotes
 {
 	string::size_type	len = src.size();
 	string	dest;
@@ -155,7 +155,13 @@ string	UnquoteShellChars(const string &src)	// Remove quotes
 					i += 2;
 					break;
 				default:
-					dest += src[i];
+					if (quoteGlob) {
+						if (strchr(charNeedGlobQuote, src[i]))
+							dest += '\\';
+						dest += src[i];
+					}
+					else
+						dest += src[i];
 			}
 			quoted = false;
 		}
